/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/create/platonic.h>


class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::VFAdj, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj, vcg::face::VFAdj, vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

class MyVertex0  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::BitFlags  >{};
class MyVertex1  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyVertex2  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Color4b, vcg::vertex::CurvatureDirf,
                                                    vcg::vertex::Qualityf, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
using namespace vcg;

    

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;
  
  tri::Icosahedron(m);
  tri::RequirePerVertexNormal(m);
  tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  tri::UpdateTopology<MyMesh>::VertexFace(m);
  
  tri::io::ExporterOFF<MyMesh>::Save(m,"Test.off");
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  
  MyMesh::FacePointer ff = &(m.face[0]);
  
  printf("face 0 is adjacent to face %lu\n", tri::Index(m,ff->FFp(0)));
  printf("face 0 is adjacent to face %lu\n", tri::Index(m,ff->FFp(1)));
  printf("face 0 is adjacent to face %lu\n", tri::Index(m,ff->FFp(2)));
  
  MyMesh::FacePointer vfP = m.vert[0].VFp();
  int vfI = m.vert[0].VFi();
  int faceCnt=0;
  while(vfP != nullptr)
  {
      printf("Face %lu \n", tri::Index(m,vfP));
      faceCnt++;
      MyMesh::FacePointer nextFp = vfP->VFp(vfI);
      int nextFi=vfP->VFi(vfI);
      vfP = nextFp;
      vfI = nextFi;      
  }
  printf("Face count %i \n",faceCnt);
  faceCnt=0;
  vcg::face::VFIterator<MyFace> vfi(&(m.vert[0])); 
  for(;!vfi.End();++vfi)
  {
      printf("Face %lu \n", tri::Index(m,vfi.F()));
      faceCnt++;
  }
  printf("Face count %i \n",faceCnt);
  
  std::vector<MyMesh::FacePointer> facePtrVec;
  std::vector<int> faceIndVec;
  
  vcg::face::VFStarVF(&(m.vert[0]),facePtrVec,faceIndVec);
  printf("Face count %i \n",facePtrVec.size());
  
  
  
  return 0;
}
